@@ -32,11 +32,11 @@
 #define DEBUG 0
 
 #include "sysincludes.h"
+#include "llong.h"
 
 #ifdef USE_FLOPPYD
 
 #define USE_FLOPPYD_BUFFERED_IO  1
-#define FLOPPYD_DEFAULT_PORT     5703
 
 #include "sysincludes.h"
 #include "grp.h"
@@ -103,14 +103,15 @@
 
 typedef unsigned char Byte;
 typedef unsigned long Dword;
-
+typedef mt_off_t Qword;
 
 #define MAX_XAUTHORITY_LENGTH    3000
 #define MAX_DATA_REQUEST         3000000
 #define BUFFERED_IO_SIZE         16348
 
+unsigned int mtools_lock_timeout=30;
 
-void serve_client(int sock, char **device_name, int n_dev);
+void serve_client(int sock, char **device_name, int n_dev, int close_stderr);
 
 
 #ifdef USE_FLOPPYD_BUFFERED_IO
@@ -118,7 +119,7 @@ typedef struct io_buffer {
 	Byte out_buffer[BUFFERED_IO_SIZE];
 	Byte in_buffer[BUFFERED_IO_SIZE];
 	
-	long in_valid;
+	unsigned long in_valid;
 	long in_start;
 	long out_valid;
 	
@@ -139,7 +140,9 @@ static io_buffer new_io_buffer (int _handle) {
 
 static void flush(io_buffer buffer) {
 	if (buffer->out_valid) {
-		write(buffer->handle, buffer->out_buffer, buffer->out_valid);
+		if(write(buffer->handle, buffer->out_buffer, buffer->out_valid) < 0) {
+			perror("floppyd flush");
+		}
 		buffer->out_valid = 0;
 	}
 }
@@ -151,7 +154,7 @@ static void free_io_buffer(io_buffer buffer) {
 
 
 static size_t buf_read (io_buffer buf, Byte* buffer, size_t nbytes) {
-	size_t rval;
+	ssize_t rval;
 	
 	if (nbytes <= buf->in_valid) {
 		memcpy(buffer, buf->in_buffer+buf->in_start, nbytes);
@@ -174,7 +177,7 @@ static size_t buf_read (io_buffer buf, Byte* buffer, size_t nbytes) {
 			rval = read(buf->handle, buf->in_buffer, 
 						BUFFERED_IO_SIZE);
 			if (rval >= 0) {
-				if (rval < nbytes) {
+				if (rval < (ssize_t) nbytes) {
 					memcpy(buffer, buf->in_buffer, rval);
 					rval += buf->in_valid;
 					buf->in_valid = buf->in_start = 0;
@@ -320,7 +323,7 @@ static char send_packet(Packet packet, io_buffer fp)
 
 static char recv_packet(Packet packet, io_buffer fp, Dword maxlength)
 {
-	int start;
+	Dword start;
 	int l;
 	Dword length = read_dword(fp);
 #if DEBUG
@@ -367,10 +370,18 @@ static void put_dword(Packet packet, int my_index, Dword val) {
 	dword2byte(val, packet->data+my_index);
 }
 
+static void put_qword(Packet packet, int my_index, Qword val) {
+	qword2byte(val, packet->data+my_index);
+}
+
 static Dword get_dword(Packet packet, int my_index) {
 	return byte2dword(packet->data+my_index);
 }	
 
+static Qword get_qword(Packet packet, int my_index) {
+	return byte2qword(packet->data+my_index);
+}	
+
 static Dword get_length(Packet packet) {
 	return packet->len;
 }
@@ -426,10 +437,14 @@ static char do_auth(io_buffer sock, int *version)
 	if(*version == FLOPPYD_PROTOCOL_VERSION_OLD) {
 		put_dword(reply, 0, AUTH_SUCCESS);
 	} else {
+		Dword cap = FLOPPYD_CAP_EXPLICIT_OPEN;
+		if(sizeof(mt_off_t) >= 8) {
+			cap |= FLOPPYD_CAP_LARGE_SEEK;
+		}
 		make_new(reply, 12);
 		put_dword(reply, 0, AUTH_SUCCESS);
 		put_dword(reply, 4, FLOPPYD_PROTOCOL_VERSION);
-		put_dword(reply, 8, FLOPPYD_CAP_EXPLICIT_OPEN);
+		put_dword(reply, 8, cap);
 	}
 	send_packet(reply, sock);
 	destroyPacket(proto_version);
@@ -481,7 +496,10 @@ static char do_auth(io_buffer sock, int *version)
 	*ptr++ = '0'; /* Display number */
 	*ptr++ = '\0';
 
-	write(fd, template, len+8);
+	if(write(fd, template, len+8) < len + 8) {
+		close(fd);
+		return 0;
+	}
 	ptr = (char *)mit_cookie->data;
 	len = mit_cookie->len;
 
@@ -496,7 +514,10 @@ static char do_auth(io_buffer sock, int *version)
 	    return 0;
 	}
 
-	write(fd, ptr, len);
+	if(write(fd, ptr, len) < len) {
+		close(fd);
+		return 0;
+	}
 	close(fd);
 
 	destroyPacket(mit_cookie);
@@ -664,8 +685,11 @@ static int bind_to_port(IPaddr_t bind_ip, short bind_port)
 	 */
 	{
 	 	int	on = 1;
-		setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, 
-				   (char *)&on, sizeof(on));
+		if(setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, 
+			      (char *)&on, sizeof(on)) < 0) {
+			perror("setsockopt");
+			exit(1);
+		}
 	}
 
 	/*
@@ -701,7 +725,8 @@ static int sockethandle_now = -1;
 /*
  * Catch alarm signals and exit.
  */
-static void alarm_signal(int a)
+static void alarm_signal(int a UNUSEDP) NORETURN;
+static void alarm_signal(int a UNUSEDP)
 {
 	if (sockethandle_now != -1) {
 		close(sockethandle_now);
@@ -715,6 +740,7 @@ static void alarm_signal(int a)
 /*
  * This is the main loop when running as a server.
  */
+static void server_main_loop(int sock, char **device_name, int n_dev) NORETURN;
 static void server_main_loop(int sock, char **device_name, int n_dev)
 {
 	struct sockaddr_in	addr;
@@ -749,7 +775,7 @@ static void server_main_loop(int sock, char **device_name, int n_dev)
 				 * Start the proxy work in the new socket.
 				 */
 #endif
-				serve_client(new_sock,device_name, n_dev);
+				serve_client(new_sock,device_name, n_dev, 0);
 				exit(0);
 #if DEBUG == 0
 		}
@@ -765,6 +791,7 @@ static void server_main_loop(int sock, char **device_name, int n_dev)
 /*
  * Print some basic help information.
  */
+static void usage(char *prog, const char *opt, int ret) NORETURN;
 static void usage(char *prog, const char *opt, int ret)
 {
 	if (opt)
@@ -800,9 +827,7 @@ int main (int argc, char** argv)
 	gid_t			run_gid = 65535;
 	char*			username = strdup("nobody");
 	int			sock;
-	int			port_is_supplied = 0;
 
-	char *server_hostname=NULL;
 	char **device_name = NULL; 
 	const char *floppy0 = "/dev/fd0";
 	int n_dev;
@@ -822,7 +847,6 @@ int main (int argc, char** argv)
 						break;
 					case 's':
 						run_as_server = 1;
-						port_is_supplied = 1;
 						bind_port = getportnum(optarg);
 						break;
 
@@ -835,7 +859,6 @@ int main (int argc, char** argv)
 					case 'b':
 						run_as_server = 1;
 						bind_ip = getipaddress(optarg);
-						server_hostname=optarg;
 						break;
 					case 'x':
 						dispName = strdup(optarg);
@@ -843,10 +866,8 @@ int main (int argc, char** argv)
 
 					case 'h':
 						usage(argv[0], NULL, 0);
-						break;
 					case '?':
 						usage(argv[0], NULL, 1);
-						break;
 				}
 		}
 
@@ -879,9 +900,7 @@ int main (int argc, char** argv)
 		/* try to find out port that we are connected to */
 		if(getsockname(0, (struct sockaddr*) &addr, &len) >= 0 && 
 		   len == sizeof(addr)) {
-			port_is_supplied = 1;
 			bind_port = ntohs(addr.sin_port);
-			server_hostname = strdup(inet_ntoa(addr.sin_addr));
 		}
 	}
 
@@ -961,11 +980,6 @@ int main (int argc, char** argv)
 					 * Handle the server main loop.
 					 */
 					server_main_loop(sock, device_name, n_dev);
-					
-					/*
-					 * Should never exit.
-					 */
-					exit(1);
 				}
 		
 		/*
@@ -985,13 +999,9 @@ int main (int argc, char** argv)
 	signal(SIGPIPE, alarm_signal);
 	/*signal(SIGALRM, alarm_signal);*/
 
-#if DEBUG == 0
-	close(2);
-	open("/dev/null", O_WRONLY);
-#endif
 	/* Starting from inetd */
 
-	serve_client(sockfd, device_name, n_dev);
+	serve_client(sockfd, device_name, n_dev, 1);
 	return 0;
 }
 
@@ -1009,14 +1019,30 @@ static void send_reply(int rval, io_buffer sock, int len) {
 	destroyPacket(reply);
 }
 
-static void cleanup(int x) {
+static void send_reply64(int rval, io_buffer sock, mt_off_t len) {
+	Packet reply = newPacket();
+
+	make_new(reply, 12);
+	put_qword(reply, 0, len);
+	if (rval == -1) {
+		put_dword(reply, 8, 0);
+	} else {
+		put_dword(reply, 8, errno);
+	}
+	send_packet(reply, sock);
+	destroyPacket(reply);
+}
+
+static void cleanup(int x UNUSEDP) NORETURN;
+static void cleanup(int x UNUSEDP) {
 	unlink(XauFileName());
 	exit(-1);
 }
 
 #include "lockdev.h"
 
-void serve_client(int sockhandle, char **device_name, int n_dev) {
+void serve_client(int sockhandle, char **device_name, int n_dev,
+		  int close_stderr) {
 	Packet opcode;
 	Packet parm;
 
@@ -1033,10 +1059,22 @@ void serve_client(int sockhandle, char **device_name, int n_dev) {
 	 */
 	{
 		int		on = 1;
-		setsockopt(sockhandle, SOL_SOCKET, 
-				   SO_KEEPALIVE, (char *)&on, sizeof(on));
+		if(setsockopt(sockhandle, SOL_SOCKET, 
+			      SO_KEEPALIVE, (char *)&on, sizeof(on)) < 0) {
+			perror("setsockopt");
+			exit(1);
+		}
+			
 	}
 
+
+#if DEBUG == 0
+	if(close_stderr) {
+		close(2);
+		open("/dev/null", O_WRONLY);
+	}
+#endif
+
 	sock = new_io_buffer(sockhandle);
 	
 	/*
@@ -1070,12 +1108,12 @@ void serve_client(int sockhandle, char **device_name, int n_dev) {
 	if(version == FLOPPYD_PROTOCOL_VERSION_OLD) {
 				/* old protocol */
 		readOnly = 0;
-		devFd = open(device_name[0], O_RDWR);
+		devFd = open(device_name[0], O_RDWR|O_LARGEFILE);
 		
 		if (devFd < 0) {
 			readOnly = 1;
 			devFd = open(device_name[0], 
-				     O_RDONLY);
+				     O_RDONLY|O_LARGEFILE);
 		}
 		if(devFd < 0) {
 			send_reply(0, sock, devFd);
@@ -1110,7 +1148,8 @@ void serve_client(int sockhandle, char **device_name, int n_dev) {
 					break;
 				}
 
-				devFd = open(device_name[dev_nr], O_RDONLY);
+				devFd = open(device_name[dev_nr],
+					     O_RDONLY | O_LARGEFILE);
 #if DEBUG
 				fprintf(stderr, "Device opened\n");
 #endif
@@ -1144,8 +1183,7 @@ void serve_client(int sockhandle, char **device_name, int n_dev) {
 #endif
 				read_packet(parm, devFd, get_dword(parm, 0));
 				send_reply(devFd, sock, get_length(parm));
-				if(get_length(parm) >= 0)
-					send_packet(parm, sock);
+				send_packet(parm, sock);
 				break;
 			case OP_WRITE:
 #if DEBUG
@@ -1170,6 +1208,24 @@ void serve_client(int sockhandle, char **device_name, int n_dev) {
 					   sock, 
 					   lseek(devFd, 0, SEEK_CUR));
 				break;
+			case OP_SEEK64:
+				if(sizeof(mt_off_t) < 8) {
+#if DEBUG
+					fprintf(stderr, "64 bit requested where not available!\n");
+#endif
+					errno = EINVAL;
+					send_reply(devFd, sock, -1);
+					break;
+				}
+#if DEBUG
+				fprintf(stderr, "SEEK64:\n");
+#endif
+				mt_lseek(devFd, 
+					 get_qword(parm,0), get_dword(parm,8));
+				send_reply64(devFd, 
+					     sock, 
+					     mt_lseek(devFd, 0, SEEK_CUR));
+				break;
 			case OP_FLUSH:
 #if DEBUG
 				fprintf(stderr, "FLUSH:\n");
