@@ -86,18 +86,20 @@ static const char *fix_mcwd(char *ans)
 }
 
 int unix_dir_loop(Stream_t *Stream, MainParam_t *mp); 
-int unix_loop(Stream_t *Stream, MainParam_t *mp, char *arg,
+int unix_loop(Stream_t *Stream UNUSEDP, MainParam_t *mp, char *arg,
 	      int follow_dir_link);
 
-static int _unix_loop(Stream_t *Dir, MainParam_t *mp, const char *filename)
+static int _unix_loop(Stream_t *Dir, MainParam_t *mp,
+		      const char *filename UNUSEDP)
 {
 	return unix_dir_loop(Dir, mp);
 }
 
-int unix_loop(Stream_t *Stream, MainParam_t *mp, char *arg, int follow_dir_link)
+int unix_loop(Stream_t *Stream UNUSEDP, MainParam_t *mp,
+	      char *arg, int follow_dir_link)
 {
 	int ret;
-	int isdir;
+	int isdir=0;
 	int unixNameLength;
 
 	mp->File = NULL;
@@ -145,7 +147,7 @@ int unix_loop(Stream_t *Stream, MainParam_t *mp, char *arg, int follow_dir_link)
 #endif
 			if(! (mp->lookupflags & ACCEPT_DIR))
 				return 0;
-			mp->File = OpenDir(Stream, arg);
+			mp->File = OpenDir(arg);
 		}
 	}
 
