@@ -1,4 +1,4 @@
-/*  Copyright 1995-2007,2009 Alain Knaff.
+/*  Copyright 1995-2007,2009,2011 Alain Knaff.
  *  This file is part of mtools.
  *
  *  Mtools is free software: you can redistribute it and/or modify
@@ -32,6 +32,10 @@
 #include "partition.h"
 #include "llong.h"
 
+#ifdef HAVE_LINUX_FS_H
+# include <linux/fs.h>
+#endif
+
 typedef struct SimpleFile_t {
     Class_t *Class;
     int refs;
@@ -146,7 +150,7 @@ static int file_write(Stream_t *Stream, char *buf, mt_off_t where, size_t len)
 	}
 }
 
-static int file_flush(Stream_t *Stream)
+static int file_flush(Stream_t *Stream UNUSEDP)
 {
 #if 0
 	DeclareThis(SimpleFile_t);
@@ -219,7 +223,7 @@ static int file_geom(Stream_t *Stream, struct device *dev,
 		}
 
 		if (boot->boot.descr >= 0xf0 &&
-		    labelBlock->dos4 == 0x29 &&
+		    has_BPB4 &&
 		    strncmp( boot->boot.banner,"2M", 2 ) == 0 &&
 		    BootP < 512 && Infp0 < 512 && InfpX < 512 && InfTm < 512 &&
 		    BootP >= InfTm + 2 && InfTm >= InfpX && InfpX >= Infp0 && 
@@ -232,17 +236,9 @@ static int file_geom(Stream_t *Stream, struct device *dev,
 				dev->ssize |= 0x80; /* is set */
 			}
 		}
-	} else if (media >= 0xf8){
-		media &= 3;
-		dev->heads = old_dos[media].heads;
-		dev->tracks = old_dos[media].tracks;
-		dev->sectors = old_dos[media].sectors;
-		dev->ssize = 0x80;
-		dev->use_2m = ~1;
-	} else {
-		fprintf(stderr,"Unknown media type\n");
-		exit(1);
-	}
+	} else
+		if(setDeviceFromOldDos(media, dev) < 0)
+			exit(1);
 
 	sectors = dev->sectors;
 	dev->sectors = dev->sectors * WORD(secsiz) / 512;
@@ -277,6 +273,20 @@ static int file_data(Stream_t *Stream, time_t *date, mt_size_t *size,
 	return 0;
 }
 
+static int file_discard(Stream_t *Stream)
+{
+#ifdef BLKFLSBUF
+	int ret;
+	DeclareThis(SimpleFile_t);
+	ret= ioctl(This->fd, BLKFLSBUF);
+	if(ret < 0)
+		perror("BLKFLSBUF");
+	return ret;
+#else
+	return 0;
+#endif
+}
+
 /* ZIP or other scsi device on Solaris or SunOS system.
    Since Sun won't accept a non-Sun label on a scsi disk, we must
    bypass Sun's disk interface and use low-level SCSI commands to read
@@ -294,8 +304,6 @@ static int file_data(Stream_t *Stream, time_t *date, mt_size_t *size,
    group called, say, "ziprw" which has rw permission on /dev/rsd5c, is fine.
  */
 
-#define MAXBLKSPERCMD 255
-
 static void scsi_init(SimpleFile_t *This)
 {
    int fd = This->fd;
@@ -455,7 +463,9 @@ static Class_t ScsiClass = {
 	file_free,
 	file_geom,
 	file_data,
-	0 /* pre-allocate */
+	0, /* pre-allocate */
+	0, /* dos-convert */
+	file_discard
 };
 
 
@@ -466,7 +476,9 @@ static Class_t SimpleFileClass = {
 	file_free,
 	file_geom,
 	file_data,
-	0 /* pre_allocate */
+	0, /* pre_allocate */
+	0, /* dos-convert */
+	file_discard
 };
 
 
@@ -656,7 +668,7 @@ APIRET rc;
 		} else {
 			*maxSize = max_off_t_seek;
 		}
-		if(This->offset > *maxSize) {
+		if(This->offset > (mt_off_t) *maxSize) {
 			close(This->fd);
 			Free(This);
 			if(errmsg)
@@ -685,7 +697,7 @@ APIRET rc;
 	This->swap = DO_SWAP( dev );
 
 	if(!(mode2 & NO_OFFSET) &&
-	   dev && (dev->partition > 4 || dev->partition < 0))
+	   dev && (dev->partition > 4))
 	    fprintf(stderr, 
 		    "Invalid partition %d (must be between 0 and 4), ignoring it\n", 
 		    dev->partition);
