@@ -1,4 +1,21 @@
-/* System includes for mtools */
+/*  Copyright 1996-1999,2001,2002,2007-2009 Alain Knaff.
+ *  This file is part of mtools.
+ *
+ *  Mtools is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Mtools is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with Mtools.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * System includes for mtools
+ */
 
 #ifndef SYSINCLUDES_H
 #define SYSINCLUDES_H
@@ -69,7 +86,7 @@ ac_cv_func_setpgrp_void=yes ../mtools/configure --build=i386-linux-gnu --host=i3
 #endif
 #endif
 
-#ifdef OS_mingw32msvc
+#ifndef HAVE_CADDR_T
 typedef void *caddr_t;
 #endif
 
@@ -87,12 +104,15 @@ typedef void *caddr_t;
 # if __GNUC__ == 2 && __GNUC_MINOR__ > 6 || __GNUC__ >= 3
 /* gcc 2.6.3 doesn't have "unused" */		/* mool */
 #  define UNUSED(x) x __attribute__ ((unused));x
+#  define UNUSEDP __attribute__ ((unused))
 # else
 #  define UNUSED(x) x
+#  define UNUSEDP /* */
 # endif
 # define NORETURN __attribute__ ((noreturn))
 #else
 # define UNUSED(x) x
+#  define UNUSEDP /* */
 # define PACKED /* */
 # define NORETURN /* */
 #endif
@@ -115,6 +135,14 @@ typedef void *caddr_t;
 
 #include <sys/types.h>
 
+#ifdef HAVE_STDINT_H
+# include <stdint.h>
+#endif
+
+#ifdef HAVE_INTTYPES_H
+# include <inttypes.h>
+#endif
+
 #ifdef HAVE_STDLIB_H
 # include <stdlib.h>
 #endif
@@ -259,6 +287,10 @@ typedef unsigned int uid_t;
 # include <malloc.h>
 #endif
 
+#ifdef HAVE_IO_H
+# include <io.h>
+#endif
+
 #ifdef HAVE_SIGNAL_H
 # include <signal.h>
 #else
@@ -277,6 +309,35 @@ typedef unsigned int uid_t;
 # endif
 #endif
 
+#ifdef HAVE_WCHAR_H
+# include <wchar.h>
+# ifndef HAVE_PUTWC
+#  define putwc(c,f) fprintf((f),"%lc",(c))
+# endif
+#else
+# define wcscmp strcmp
+# define wcscasecmp strcasecmp
+# define wcsdup strdup
+# define wcslen strlen
+# define wcschr strchr
+# define wcspbrk strpbrk
+# define wchar_t char
+# define putwc putc
+#endif
+
+#ifdef HAVE_WCTYPE_H
+# include <wctype.h>
+#else
+# define towupper(x) toupper(x)
+# define towlower(x) tolower(x)
+# define iswupper(x) isupper(x)
+# define iswlower(x) islower(x)
+# define iswcntrl(x) iscntrl(x)
+#endif
+
+#ifdef HAVE_LOCALE_H
+# include <locale.h>
+#endif
 
 #ifdef USE_FLOPPYD
 
@@ -351,6 +412,9 @@ typedef unsigned int uid_t;
 extern char *strdup(const char *str);
 #endif /* HAVE_STRDUP */
 
+#ifndef HAVE_STRNDUP
+extern char *strndup(const char *s, size_t n);
+#endif /* HAVE_STRDUP */
 
 #ifndef HAVE_MEMCPY
 extern char *memcpy(char *s1, const char *s2, size_t n);
@@ -409,6 +473,22 @@ int strncasecmp(const char *s1, const char *s2, size_t n);
 char *getpass(const char *prompt);
 #endif
 
+#ifdef HAVE_WCHAR_H
+
+# ifndef HAVE_WCSDUP
+wchar_t *wcsdup(const wchar_t *wcs);
+# endif
+
+# ifndef HAVE_WCSCASECMP
+int wcscasecmp(const wchar_t *s1, const wchar_t *s2);
+# endif
+
+# ifndef HAVE_WCSNLEN
+size_t wcsnlen(const wchar_t *wcs, size_t l);
+# endif
+
+#endif
+
 #if 0
 #ifndef HAVE_BASENAME
 const char *basename(const char *filename);
